#include "testlib.h"
#include <unordered_set>

std::vector<std::unordered_set<int>> graph;
int n;

void check_1() {
	int cycle1_len = ouf.readInt();
	if(cycle1_len < 2 || cycle1_len > n) quitf(_wa, "bad cycle length");
	if(cycle1_len % 2 == 0) quitf(_wa, "expected odd cycle length");

	int cycle2_len = ouf.readInt();
	if(cycle2_len < 2 || cycle2_len > n) quitf(_wa, "bad cycle length");
	if(cycle2_len % 2 == 0) quitf(_wa, "expected odd cycle length");

	// todo do we need to ban non-simple cycles?? they don't help you in any way
	std::vector<std::pair<int,int>> cycles[2];
	for(int i = 0; i < 2; i++) {
		int len = i == 1 ? cycle2_len : cycle1_len;
		int first_node = ouf.readInt();
		int prev_node = first_node;
		for(int j = 1; j < len; j++) {
			int this_node = ouf.readInt();
			int a = std::min(this_node, prev_node);
			int b = std::max(this_node, prev_node);
			cycles[i].emplace_back(a, b);
			prev_node = this_node;
		}
		int a = std::min(first_node, prev_node);
		int b = std::max(first_node, prev_node);
		cycles[i].emplace_back(a, b);

		for(auto& p : cycles[i]) {
			if(p.first < 1 || p.first > n) quitf(_wa, "cycle contains invalid node");
			if(graph[p.first].count(p.second) == 0) quitf(_wa, "cycle contains invalid edge");
		}
	}

	// set of edges of cycle1. (start << 32) + end, because pair<int,int> isn't hashable
	std::unordered_set<uint64_t> seen;
	for(auto& p : cycles[0]) {
		seen.emplace((uint64_t)p.first << 32 | p.second);
	}
	for(auto& p : cycles[1]) {
		if(seen.count((uint64_t)p.first << 32 | p.second) != 0) {
			quitf(_wa, "both cycles contain a common edge");
		}
	}
	quitf(_ok, "translate:success");
}

void check_2() {
	std::vector<unsigned char> colors;
	colors.push_back(0);
	for(int i = 0; i < n; i++) {
		int c = ouf.readInt();
		if(c < 1 || c > 4) quitf(_pe, "invalid color");
		colors.push_back(c);
	}
	for(int i = 1; i <= n; i++) {
		for(auto neighbor : graph[i]) {
			if(colors[neighbor] == colors[i]) quitf(_wa, "bad coloring");
		}
	}
	quitf(_ok, "translate:success");
}

int main(int argc, char** argv) {
	if (argc < 3) {
		std::cerr << "usage: <input-file> <output-file>" << std::endl;
		return 1;
	} else if (argc == 3) {
		// this checker doesn't use the answer file
		// shortcut
		char *nargv [4] = {argv[0], argv[1], argv[2], argv[2]};
		registerTestlibCmd(4, nargv);
	} else {
		registerTestlibCmd(argc, argv);
	}

	n = inf.readInt();
	int m = inf.readInt();

	graph.resize(n+1);
	for(int i = 0; i < m; i++) {
		int u = inf.readInt();
		int v = inf.readInt();
		graph[v].insert(u);
		graph[u].insert(v);
	}

	int subprob = ouf.readInt();
	if(subprob == 1) {
		check_1();
	} else if(subprob == 2) {
		check_2();
	} else {
		quitf(_pe, "Expected 1 or 2 on first line of output");
	}
}
