#include <algorithm>
#include <iostream>
#include <queue>
#include <unordered_set>
#include <vector>

// idee:
// hakkame 2 värviga värvima
// kui leiame paaritu tsükli, siis võtame ta välja ja jätame meelde
// (sellega võib tekkida mitu sidususkomponenti)
// kui leiame teise paaritu tsükli, siis tagastame leitud tsüklid
// kui ei leia, oleme ülejäänud graafi 2 värviga värvinud
// k.a. kõik tsükli tipud (mis võivad eri komponentides olla)
// jätame tsükli esimese tipu oma värvi
// edasi värvime tsükli tipud järjest vaheldumisi 3./4. värvi

std::vector<std::unordered_set<int>> graph;
// 0 - not visited, 1/2 - bipartite coloring
std::vector<int> coloring;

std::vector<int> find_cycle(int root) {
	std::queue<int> q;
	std::vector<int> parent(graph.size(), -1);
	q.push(root); // root node
	coloring[root] = 1;
	while(!q.empty()) {
		int node = q.front();
		q.pop();
		int nodecolor = coloring[node];
		for(auto neigh : graph[node]) {
			if(neigh != parent[node] && coloring[neigh] == nodecolor) {
				// we have an odd cycle: 1 - ... - node - neigh - ... - 1
				// reconstruct it and remove redundant nodes
				std::vector<int> path1;
				while(node != -1) { path1.push_back(node); node = parent[node]; }
				std::vector<int> path2;
				while(neigh != -1) { path2.push_back(neigh); neigh = parent[neigh]; }
				// delete redundant common nodes
				int last = path1[path1.size()-1];
				while(path1[path1.size() - 1] == path2[path2.size() - 1]) {
					last = path1[path1.size()-1];
					path1.pop_back(); path2.pop_back();
				}
				// re-add the connecting node
				path1.push_back(last);
				// join the paths together
				std::move(path2.rbegin(), path2.rend(), std::back_inserter(path1));
				return path1;
			} else if(coloring[neigh] == 0) {
				// 1 -> 2, 2 -> 1
				coloring[neigh] = 3 - nodecolor;
				parent[neigh] = node;
				q.push(neigh);
			} else {
				// already visited/queued different color neighbor
			}
		}
	}
	return {};
}

int main() {
	std::ios::sync_with_stdio(false);
	int n, m;
	std::cin >> n >> m;
	graph.resize(n);
	coloring.resize(n);
	for(int i = 0; i < m; i++) {
		int u,v;
		std::cin >> u >> v;
		graph[v-1].insert(u-1);
		graph[u-1].insert(v-1);
	}
	std::vector<int> cyc1 = find_cycle(0);
	if(cyc1.empty()) {
		// bipartite - successfully 2-colored
		std::cout << "2\n";
		for(int color : coloring) std::cout << color << ' ';
		std::cout << '\n';
		return 0;
	} else {
		for(int i = 0; i < cyc1.size(); i++) {
			int a = cyc1[i];
			int b = cyc1[(i + 1) % cyc1.size()];
			graph[a].erase(b);
			graph[b].erase(a);
		}
		coloring.clear();
		coloring.resize(n);
		for(int cycle_el : cyc1) {
			if(coloring[cycle_el] != 0) continue;
			std::vector<int> cyc2 = find_cycle(cycle_el);
			if(!cyc2.empty()) {
				// found 2 cycles
				std::cout << "1\n";
				std::cout << cyc1.size() << ' ' << cyc2.size() << '\n';
				for(int elem : cyc1) std::cout << elem+1 << ' ';
				std::cout << '\n';
				for(int elem : cyc2) std::cout << elem+1 << ' ';
				std::cout << '\n';
				return 0;
			}
			// otherwise, this component was colored with just 1 and 2
		}
		// everything should have a coloring now
		for(int i = 1; i < cyc1.size(); i++) {
			coloring[cyc1[i]] = (i & 1) + 3;
		}
		std::cout << "2\n";
		for(int color : coloring) std::cout << color << ' ';
		std::cout << '\n';
		return 0;
	}
}
