// quadratic solution

#include <iostream>
#include <algorithm>
#include <vector>

using namespace std;

typedef long long ll;

const int FLAT = 0;
const int RISE = 1;
const int FALL = 2;

const string names [3] = {"TASANE", "TOUSEV", "LASKUV"};

const int INF = 1e9 + 5;

struct Cell {
  int value = INF, last_i = -1, last_k = -1;
};

bool operator< (Cell p, Cell q) {
  if (p.value != q.value)
    return p.value < q.value;

  return false;
}

struct Phase {
  int start, end, type;
};

int main () {
  ios::sync_with_stdio(false);
  cin.tie(0);

  int n, m, p, q;
  cin >> n >> m >> p >> q;

  vector<int> arr (n + 1);
  for (int i = 0; i <= n; i++)
    cin >> arr[i];

  int last_fall = -1, last_rise = -1;
  vector<vector<Cell>> dp (n + 1, vector<Cell> (3, Cell()));
  for (int k = 0; k < 3; k++)
    dp[0][k].value = 0;

  for (int i = 1; i <= n; i++) {
    for (int j = i - 1; j >= 0 && j > min(last_fall, last_rise); j--) {
      // rise
      if ((ll) q * (arr[i] - arr[j]) >= (ll) p * (i - j) &&
          i - j >= m && j > last_fall) {
        last_rise = max(last_rise, j);
        for (int k = 0; k < 3; k++) {
          if (k == RISE)
            continue;

          dp[i][RISE] = min(dp[i][RISE], Cell {.value = dp[j][k].value + 1, .last_i = j, .last_k = k});
        }
      }

      // fall
      if ((ll) q * (arr[i] - arr[j]) <= - (ll) p * (i - j) &&
          i - j >= m && j > last_rise) {
        last_fall = max(last_fall, j);
        for (int k = 0; k < 3; k++) {
          if (k == FALL)
            continue;

          dp[i][FALL] = min(dp[i][FALL], Cell {.value = dp[j][k].value + 1, .last_i = j, .last_k = k});
        }
      }

      // flat
      if (-p < (ll) q * (arr[i] - arr[j]) && (ll) q * (arr[i] - arr[j]) < p &&
          j > last_rise && j > last_fall) {
        for (int k = 0; k < 3; k++) {
          if (k == FLAT)
            continue;

          dp[i][FLAT] = min(dp[i][FLAT], Cell {.value = dp[j][k].value + 1, .last_i = j, .last_k = k});
        }
      }
    }
  }

  Cell best;
  for (int k = 0; k < 3; k++) {
    best = min(best, Cell {.value = dp[n][k].value, .last_i = n, .last_k = k});
  }

  if (best.value == INF) {
    cout << -1 << '\n';
    return 0;
  }

  vector<Phase> ans;
  int cur_i = best.last_i, cur_k = best.last_k;
  while (cur_i > 0) {
    auto nxt = dp[cur_i][cur_k];
    ans.push_back(Phase {.start = nxt.last_i, .end = cur_i, .type = cur_k});
    cur_i = nxt.last_i;
    cur_k = nxt.last_k;
  }

  reverse(ans.begin(), ans.end());

  cout << (int) ans.size() << '\n';
  for (auto phase : ans) {
    cout << phase.start << " " << phase.end << " " << names[phase.type] << '\n';
  }
}
