// wrong greedy O(n**2) solution
// uses the fall_stop and rise_stop from the full solution to do something similar
// to wrong_greedy.cpp

#include <iostream>
#include <vector>
#include <optional>
#include <bit>
#include <map>
#include <algorithm>

using namespace std;

typedef long long ll;

const int FLAT = 0;
const int RISE = 1;
const int FALL = 2;

const string names [3] = {"TASANE", "TOUSEV", "LASKUV"};

struct Phase {
  int start, end, type;
};

template<typename T, typename Compare>
class MonoStack {
  int idx;
  vector<pair<T, int>> stack;
  Compare cmp;

public:
  MonoStack () : idx(0), stack(), cmp() {}
  
  void push (T x) {
    while (!stack.empty() && cmp(x, stack.back().first))
      stack.pop_back();
    
    stack.emplace_back(x, idx);
    idx++;
  }

  // topmost x such that cmp(x, y) holds, and its original index
  optional<pair<T, int>> top (T y) {
    if (stack.empty() || !cmp(stack[0].first, y))
      return {};

    int ret = 0;
    for (int k = 1 << (bit_width(stack.size()) - 1); k != 0; k >>= 1)
      if (ret + k < (int) stack.size() && cmp(stack[ret + k].first, y))
        ret += k;

    return stack[ret];
  }
};

int main () {
  ios::sync_with_stdio(false);
  cin.tie(0);

  int n, m, p, q;
  cin >> n >> m >> p >> q;

  vector<int> arr (n + 1);
  for (int i = 0; i <= n; i++)
    cin >> arr[i];

  vector<ll> rise_pot (n + 1), fall_pot (n + 1), flat_pot (n + 1);
  for (int i = 0; i <= n; i++) {
    rise_pot[i] = (ll) q * arr[i] - (ll) p * i;
    fall_pot[i] = (ll) q * arr[i] + (ll) p * i;
    flat_pot[i] = (ll) q * arr[i];
  }

  // first, compute for each r, the minimum allowable l (for certain sections)
  vector<int> rise_stop (n + 1, 0), fall_stop (n + 1, 0);
  MonoStack<ll, less_equal<ll>> rise_stack;
  MonoStack<ll, greater_equal<ll>> fall_stack;
  for (int i = 0; i <= n; i++) {
    if (i >= m) {
      rise_stack.push(rise_pot[i - m]);
      fall_stack.push(fall_pot[i - m]);
    }

    if (i != 0) {
      rise_stop[i] = rise_stop[i - 1];
      fall_stop[i] = fall_stop[i - 1];
    }

    // discover the new shortest falling & rising sections
    auto rx = rise_stack.top(rise_pot[i]);
    auto fx = fall_stack.top(fall_pot[i]);

    // if both are present, the one with higher second component must be processed first
    bool fall_first = false;
    if (rx && fx) {
      if (fx.value().second > rx.value().second)
        fall_first = true;
    }

    if (fall_first)
      if (fx && fx.value().second >= rise_stop[i])
        fall_stop[i] = max(fall_stop[i], fx.value().second + 1);

    if (rx && rx.value().second >= fall_stop[i])
      rise_stop[i] = max(rise_stop[i], rx.value().second + 1);

    if (!fall_first)
      if (fx && fx.value().second >= rise_stop[i])
        fall_stop[i] = max(fall_stop[i], fx.value().second + 1);
  }

  // construct the solution from the end since that's how the fall_stop and rise_stop work right now
  int i = n, cur_k = -1;
  vector<Phase> ans;
  while (i != 0) {
    pair<int, int> best = make_pair(i, -1);

    for (int j = i - 1; j >= 0; j--) {
      // flat
      if (cur_k != FLAT && j >= rise_stop[i] && j >= fall_stop[i] &&
          abs((ll) q * (arr[i] - arr[j])) < p)
        best = min(best, make_pair(j, FLAT));

      // rise
      if (cur_k != RISE && j >= fall_stop[i] && j <= i - m &&
          (ll) q * (arr[i] - arr[j]) >= (ll) p * (i - j))
        best = min(best, make_pair(j, RISE));

      // fall
      if (cur_k != FALL && j >= rise_stop[i] && j <= i - m &&
          (ll) q * (arr[i] - arr[j]) <= - (ll) p * (i - j))
        best = min(best, make_pair(j, FALL));
    }

    if (best.first == i) {
      cout << -1 << '\n';
      return 0;
    }

    ans.push_back(Phase {.start = best.first, .end = i, .type = best.second});
    i = best.first;
    cur_k = best.second;
  }

  reverse(ans.begin(), ans.end());

  cout << (int) ans.size() << '\n';
  for (auto phase : ans) {
    cout << phase.start << " " << phase.end << " " << names[phase.type] << '\n';
  }
}
